'use strict';

/** @jsx createElement */
import { createElement, PropTypes } from 'rax';
import { isWeb, isWeex } from 'nuke-env';
import { calcTextSize } from 'nuke-helper';

import { getText, genEventObject } from '../mods/util';
import InputComponent from '../mods/input-component';

class NoStyleInput extends InputComponent {
  constructor(props, context) {
    super(props);
    this.fixedFont = context.commonConfigs && context.commonConfigs.fixedFont;
    if ('fixedFont' in props) {
      this.fixedFont = props.fixedFont;
    }
    this.getRef = this.getRef.bind(this);
    this.setNativeFormatRule = this.setNativeFormatRule.bind(this);
    this.inputHandler = this.inputHandler.bind(this);
    this.returnHandler = this.returnHandler.bind(this);
  }
  shouldComponentUpdate(nextProps) {
    if (nextProps.value !== null) {
      return (
        nextProps.value !== this.props.value ||
        nextProps.readOnly !== this.props.readOnly ||
        nextProps.type !== this.props.type ||
        nextProps.disabled !== this.props.disabled ||
        nextProps.maxLength !== this.props.maxLength ||
        nextProps.placeholder !== this.props.placeholder
      );
    }
    return (
      typeof nextProps.value !== typeof this.props.value ||
      nextProps.readOnly !== this.props.readOnly ||
      nextProps.type !== this.props.type ||
      nextProps.disabled !== this.props.disabled ||
      nextProps.maxLength !== this.props.maxLength ||
      nextProps.placeholder !== this.props.placeholder
    );
  }
  setNativeFormatRule(rules) {
    this.refs.input.setTextFormatter(rules);
  }

  getRef() {
    return this.refs.input;
  }
  returnHandler(e) {
    if (isWeb) {
      if (e.keyCode === 13) {
        this.trigger('onReturn', genEventObject(e));
      }
    } else {
      this.trigger('onReturn', genEventObject(e));
    }
  }
  inputHandler(e) {
    this.trigger('onInput', getText(e), genEventObject(e));
  }
  getInputStyle() {
    const {
      multiple,
      disabled,
      readOnly,
      inputStyle,
      style,
      themeStyle,
    } = this.props;
    const elementStyle = Object.assign(
      {},
      themeStyle['input-inner'],
      themeStyle[`${multiple ? 'multiple' : 'single'}-input`],
      readOnly ? themeStyle.readonly : {},
      disabled
        ? themeStyle[`${multiple ? 'multiple' : 'single'}-disabled-input`]
        : {},
      isWeb ? themeStyle['web-normalize'] : {},
      inputStyle,
      style
    );
    return this.fixedFont ? calcTextSize(elementStyle) : elementStyle;
  }
  getValue() {
    if (isWeb) {
      return this.refs.input.value;
    }
    return this.refs.input.attr.value;
  }
  render() {
    const {
      value,
      // onChange,
      onFocus,
      onInput,
      onBlur,
      readOnly,
      disabled,
      style = {},
      maxLength,
      multiple,
      returnKeyType,
      autoFocus,
      fixedFont,
      rows,
      type,
      placeholderColor,
      ...others
    } = this.props;

    const attrs = {
      onChange: this.changeHandler,
      onInput: this.inputHandler,
      onFocus: this.focusHandler,
      onBlur: this.blurHandler,
      disabled,
      rows,
      returnKeyType,
      maxlength: maxLength,
      readOnly,
      autofocus: autoFocus,
      placeholderColor,
      type: isWeex && multiple ? 'text' : type, // force type="text" in weex multiple mode to avoid bugs http://dotwe.org/weex/6a75d9164c4186c9a9d46b51daed6b2e
      style: this.getInputStyle(),
    };
    if ('value' in this.props) {
      attrs.value = this.props.value;
    }
    if (isWeb) {
      attrs.onKeyUp = this.returnHandler;
      // keys that should not be passed
      ['readOnly', 'autofocus'].map((item) => {
        if (item in attrs && !attrs[item]) {
          delete attrs[item];
        }
      });
    } else {
      attrs.onReturn = this.returnHandler;
      attrs['data-placeholder-color'] = placeholderColor;
      // in multiple mode, passing returnKeyType to weex will get single line input
      if (multiple && !this.props.onReturn) {
        delete attrs.onReturn;
        delete attrs.returnKeyType;
      }
    }
    let inputElement;
    if (multiple) {
      if (isWeb) {
        inputElement = (
          <textarea {...others} {...attrs} ref="input">
            {attrs.value}
          </textarea>
        );
      } else {
        inputElement = <textarea {...others} {...attrs} ref="input" />;
      }
    } else {
      // if ('height' in elementStyle) {
      //   delete elementStyle.height;
      // }
      inputElement = <input {...others} {...attrs} ref="input" />;
    }
    return inputElement;
  }
}

NoStyleInput.contextTypes = {
  androidConfigs: PropTypes.any,
};

export default NoStyleInput;
