'use strict';
/** @jsx createElement */
import { Component, createElement, cloneElement, findDOMNode } from 'rax';
import View from 'nuke-view';
import Env from 'nuke-env';
import Transition from 'nuke-transition';
import Dimensions from 'nuke-dimensions';
const { isWeb, isWeex, appInfo } = Env;

class Mask extends Component {
  constructor(props) {
    super(props);
    this.initFlag = false;
    let visible = true;

    if ('defaultVisible' in props) {
      visible = props.defaultVisible;
    }
    this.state = {
      visible: visible
    };
  }
  shouldComponentUpdate(nextProps, nextState) {
    if (!this.initFlag) {
      this.initFlag = true;
      return (
        nextProps.style !== this.props.style ||
        nextState.visible !== this.state.visible
      );
    }
    return true;
  }
  componentDidMount() {
    if (this.state.visible) {
      if (this.props.animate) {
        this.animateShow();
      } else {
        this.show();
      }
    }
  }
  show() {
    this.setState({ visible: true });
    this.props.onShow && this.props.onShow();
  }
  change(e) {
    this.setState({ visible: e.visible || !this.state.visible });
    this.props.onVisibleChanged && this.props.onVisibleChanged(e);
  }
  componentDidUpdate(prevProps, prevState) {
    if (this.state.visible && this.props.animate) {
      this.animateShow();
    }
  }
  animateShow() {
    const box = findDOMNode(this.maskNode);
    const { effect, duration } = this.props;
    // add setTimeout to avoid bugs
    setTimeout(() => {
      if (box) {
        Transition(
          box,
          { opacity: 1 },
          {
            timingFunction: effect,
            duration: duration
          },
          () => {
            this.props.onShow && this.props.onShow();
          }
        );
      }
    }, 10);
  }
  hide() {
    const { animate, effect, duration } = this.props;
    if (animate) {
      const box = this.maskNode;
      if (box) {
        Transition(
          box,
          { opacity: 0 },
          {
            timingFunction: effect,
            duration: duration
          },
          () => {
            this.setState({ visible: false });
            this.props.onHide && this.props.onHide();
          }
        );
      }
    } else {
      this.setState({
        visible: false
      });
      this.props.onHide && this.props.onHide();
    }
  }
  maskPress(e) {
    if (this.props.maskClosable) {
      this.hide();
    }
    this.props.onMaskPress && this.props.onMaskPress();
  }
  emptyEvent(e) {
    // EmptyEvent consumes events in android, so it would not be bubbled to mask press.
    if (isWeb) {
      e.stopPropagation();
    }
  }
  getChildren() {
    const { children, contentStyle = {} } = this.props;
    if (!children) return null;
    if (!Array.isArray(children)) {
      return cloneElement(children, {
        onClick: e => {
          this.emptyEvent(e);
        }
      });
    }
    if (children.length === 1) {
      return cloneElement(children[0], {
        onClick: e => {
          this.emptyEvent(e);
        }
      });
    } else {
      return (
        <View
          style={contentStyle}
          x="autowrap"
          onClick={e => this.emptyEvent(e)}
        >
          {children}
        </View>
      );
    }
  }
  render() {
    const { visible } = this.state;
    const {
      content,
      children,
      style = {},
      onVisibleChanged,
      onClick,
      noPress = false,
      ...others
    } = this.props;

    if (isWeex) {
      let nativeAttrs = {
        ...others,
        style: [appInfo.platform === 'iOS' ? styles.maskNativeIOS : {}, style],
        onClick: e => this.maskPress(e),
        onVisibleChanged: e => this.change(e)
      };
      if (noPress) {
        delete nativeAttrs.onClick;
      }
      let maskWeex = visible ? (
        <mask
          ref={n => {
            this.maskNode = findDOMNode(n);
          }}
          {...nativeAttrs}
        >
          {this.getChildren()}
        </mask>
      ) : null;
      return maskWeex;
    } else {
      return visible ? (
        <div
          {...others}
          ref={n => {
            this.maskNode = findDOMNode(n);
          }}
          style={[styles.maskWeb, style]}
          onClick={e => this.maskPress(e)}
        >
          {this.getChildren()}
        </div>
      ) : null;
    }
  }
}
const styles = {
  // In Android width & height & position will be auto calculated while in iOS won't.
  maskNativeIOS: {
    position: 'absolute',
    width: 750,
    // To get fullscreen mask , we set screen height
    height: Math.round(Dimensions.get('screen').height / window.devicePixelRatio) + 'wx'
  },

  maskWeb: {
    display: 'flex',
    boxSizing: 'border-box',
    flexDirection: 'column',
    alignContent: 'flex-start',
    flexShrink: 0,
    position: 'fixed',
    top: 0,
    left: 0,
    right: 0,
    bottom: 0,
    zIndex: 100
  },
  transparent: {
    opacity: 0
  },
  shown: {
    opacity: 1
  }
};
Mask.propTypes = {

};
Mask.defaultProps = {
  animate: true,
  effect: 'ease-in-out',
  duration: 200,
  contentStyle: {},
}
export default Mask;
