const webpack = require('webpack');
const path = require('path');
const WebpackDevServer = require('webpack-dev-server');
const FallbackPort = require('fallback-port');
const HtmlWebpackPlugin = require('html-webpack-plugin');
const inquirer = require('inquirer');
const ip = require('ip');
inquirer.registerPrompt('checkbox-plus', require('inquirer-checkbox-plus-prompt'));
const fuzzy = require('fuzzy');
const RaxPlugin = require('rax-webpack-plugin');

const log = require('../utils/log');

const fallback = new FallbackPort(9999);
const fileUtils = require('../utils/file');

/**
 * @description lauch dev server for package debug
 * @param {*} cwd  current process path
 * @param {*} args pass by process args
 */
module.exports = async function (cwd, args) {
  process.env.NODE_ENV = 'development';
  process.env.BABEL_ENV = 'development';
  const webpackConfig = require(path.join(cwd, 'scripts/webpack/docs.config'));
  const packages = fileUtils.getPackages(cwd);

  const selectPackages = await inquirer.prompt({
    type: 'checkbox-plus',
    name: 'entrys',
    message: "select witch components you'd like to debug",
    pageSize: 10,
    highlight: true,
    searchable: true,
    default: [],
    source(answersSoFar, input) {
      input = input || '';

      return new Promise((resolve) => {
        const packIndexs = packages.map(pkg => pkg.name);
        const fuzzyResult = fuzzy.filter(input, packIndexs);
        const data = fuzzyResult.map(element => element.original);
        resolve(data);
      });
    },
  });

  webpackConfig.entry = fileUtils.getDocWebpackEntry(cwd, selectPackages.entrys);
  webpackConfig.resolve.alias = fileUtils.getDocAlias(cwd, selectPackages.entrys);

  // use react as render framewrok
  if (args && args.length > 0 && args[0] === 'react') {
    process.env.BABEL_ENV = 'development:react';
    // const rax2reactPlugin = JSON.stringify({
    //   plugins: [path.resolve(cwd, './packages/babel-plugin-transform-rax-to-react/src/index.js')],
    // });
    // webpackConfig.module.rules.forEach((rule) => {
    //   if (rule.loader === 'babel-loader') {
    //     rule.query = rax2reactPlugin;
    //   }
    //   if (rule.use) {
    //     rule.use.forEach((eachLoader) => {
    //       if (eachLoader.loader === 'babel-loader') {
    //         eachLoader.query = rax2reactPlugin;
    //       }
    //     });
    //   }
    // });
  } else {
    webpackConfig.plugins.push(
      new RaxPlugin({
        target: 'bundle',
      })
    );
  }
  if (args && args.length > 1 && args[1] === 'node') {
    webpackConfig.target = 'node';
    webpackConfig.plugins.push(
      new RaxPlugin({
        target: 'umd',
      })
    );
  }
  // @hack: compat with atool-doc-loader only support webpack1
  webpackConfig.plugins.push(
    new webpack.LoaderOptionsPlugin({
      options: {
        cwd,
        demoSource: '',
        entry: webpackConfig.entry,
      },
    }),

    new HtmlWebpackPlugin({
      title: 'Nuke debugger',
      filename: 'debugger.html',
      template: path.resolve(__dirname, '../template/debugger.ejs'),
      inject: false,
      data: {
        webpackConfig,
        selectPackages,
        packages,
      },
    })
  );

  const compiler = webpack(webpackConfig);

  new webpack.ProgressPlugin().apply(compiler);

  const proxySetting = [];
  const proxy = {};

  Object.keys(webpackConfig.entry).forEach((item) => {
    proxySetting.push(`${item}.html`);
  });
  proxySetting.forEach((api) => {
    proxy[webpackConfig.output.publicPath + api] = `http://127.0.0.1:${fallback.port}/packages/`;
  });
  const server = new WebpackDevServer(compiler, {
    publicPath: webpackConfig.output.publicPath,
    quiet: false,
    noInfo: true,
    stats: {
      colors: true,
    },
    proxy,
    overlay: true,
    disableHostCheck: true,
    headers: {
      'Access-Control-Allow-Origin': '*',
    },
  });
  const ipAddress = ip.address();
  server.listen(fallback.port, '0.0.0.0', (err) => {
    if (err) {
      console.error(err.message || err);
      process.exit(1);
    }
    log.info(`The develop platform page address is http://${ipAddress}:${fallback.port}/debugger.html`);
  });
};
