const babylon = require('babylon');
const traverse = require('babel-traverse').default;
const types = require('babel-types');
const generate = require('babel-generator').default;
const codeFrame = require('babel-code-frame');

/* eslint-disable new-cap */

module.exports = function (options, inputSource, sourceMapOption) {
  let hasPlatformSpecified = false;
  let specified;

  const platformMap = {
    weex: 'isWeex',
    web: 'isWeb',
    node: 'isNode',
    reactnative: 'isReactNative',
    react: 'isWeb',
  };

  let ast;

  function objectExpressionMethod(platformName) {
    const properties = [];

    Object.keys(platformMap).forEach((p) => {
      properties.push(types.objectProperty(types.Identifier(platformMap[p]), types.booleanLiteral(p === platformName)));
    });

    return types.objectExpression(properties);
  }

  function variableDeclarationMethod(name, value) {
    return types.VariableDeclaration('const', [
      types.variableDeclarator(types.Identifier(name), types.BooleanLiteral(value)),
    ]);
  }

  try {
    ast = babylon.parse(inputSource, {
      sourceType: 'module',
      plugins: ['*'],
    });
  } catch (err) {
    if (err instanceof SyntaxError) {
      err.lineNumber = err.loc.line;
      err.column = err.loc.column + 1;

      // remove trailing "(LINE:COLUMN)" acorn message and add in esprima syntax error message start
      err.message = `Line ${err.lineNumber}: ${
        err.message.replace(/ \((\d+):(\d+)\)$/, '')
        // add codeframe
      }\n\n${codeFrame(inputSource, err.lineNumber, err.column, { highlightCode: true })}`;
    }

    throw err;
  }

  traverse(ast, {
    enter() {
      specified = new Array();
      if (typeof platformMap[options.platform] !== 'undefined') {
        hasPlatformSpecified = true;
      }
    },
    BinaryExpression(path) {
      // 对callNative进行替换
      if (
        hasPlatformSpecified &&
        path.node.left &&
        path.node.left.type === 'UnaryExpression' &&
        path.node.left.operator === 'typeof'
      ) {
        const currentPlatform = platformMap[options.platform];

        if (path.node.left.argument && path.node.left.argument.name === 'callNative') {
          let boolFlag = true;
          // typeof callNative === 'function'
          if (path.node.right && path.node.right.value === 'function') {
            if (path.node.operator === '===' || path.node.operator === '==') {
              if (currentPlatform !== 'isWeex') {
                boolFlag = false;
              } else {
                boolFlag = true;
              }
              // isWeb = typeof callNative!=='function'
            } else if (currentPlatform !== 'isWeex') {
              boolFlag = true;
            } else {
              boolFlag = false;
            }
          } else if ((path.node.operator === '===' || path.node.operator === '==') && currentPlatform === 'isWeex') {
            // typeof callNative === 'undefined'
            boolFlag = false;
          } else {
            boolFlag = true;
          }
          path.replaceWith(types.booleanLiteral(boolFlag));
          return;
        }

        // 对WXEnvironment进行替换 typeof WXEnvironment === 'object'
        if (path.node.left.argument && path.node.left.argument.name === 'WXEnvironment') {
          let boolFlag = true;
          if (path.node.right && path.node.right.value === 'object') {
            if (path.node.operator === '===' || path.node.operator === '==') {
              if (currentPlatform !== 'isWeex') {
                boolFlag = false;
              }
            } else if (currentPlatform === 'isWeex') {
              boolFlag = false;
            }
          }
          path.replaceWith(types.booleanLiteral(boolFlag));
          return;
        }

        return;
      }
    },
    CallExpression(path) {
      const { node } = path;

      if (
        hasPlatformSpecified &&
        node.callee.name === 'require' &&
        node.arguments[0] &&
        options.name.indexOf(node.arguments[0].value) !== -1
      ) {
        path.replaceWith(objectExpressionMethod(options.platform));
      }
    },
    ImportDeclaration(path) {
      const { node } = path;

      if (options.name.indexOf(node.source.value) !== -1) {
        node.specifiers.forEach((spec) => {
          if (spec.type === 'ImportNamespaceSpecifier') {
            specified.push({
              local: spec.local.name,
              imported: '*',
            });
          } else {
            specified.push({
              local: spec.local.name,
              imported: spec.imported.name,
            });
          }
        });

        if (hasPlatformSpecified) {
          specified.forEach((specObj) => {
            if (specObj.imported === '*') {
              path.insertAfter(
                types.VariableDeclaration('const', [
                  types.variableDeclarator(types.Identifier(specObj.local), objectExpressionMethod(options.platform)),
                ])
              );
            } else {
              const newNodeInit = specObj.imported === platformMap[options.platform];
              let newNode = variableDeclarationMethod(specObj.imported, newNodeInit);

              path.insertAfter(newNode);

              // Support custom alise import:
              // import { isWeex as iw } from 'universal-env';
              // const isWeex = true;
              // const iw = true;
              if (specObj.imported !== specObj.local) {
                newNode = variableDeclarationMethod(specObj.local, newNodeInit);
                path.insertAfter(newNode);
              }
            }
          });

          path.remove();
        }
      }
    },
  });

  return generate(
    ast,
    Object.assign(
      {
        sourceMaps: true,
        sourceFileName: 'inline',
        sourceMapTarget: 'inline',
      },
      sourceMapOption
    ),
    inputSource
  );
};
